@@ -14,6 +14,7 @@
 #include "settings.h"
 #include "utils.h"
 #include "rules.h"
+#include "option_parser.h"
 
 #define FDN_PATH "/org/freedesktop/Notifications"
 #define FDN_IFAC "org.freedesktop.Notifications"
@@ -100,6 +101,10 @@ static const char *introspection_xml =
     "            <annotation name=\"org.freedesktop.DBus.Property.EmitsChangedSignal\" value=\"true\"/>"
     "        </property>"
 
+    "        <property name=\"pauseLevel\" type=\"u\" access=\"readwrite\">"
+    "            <annotation name=\"org.freedesktop.DBus.Property.EmitsChangedSignal\" value=\"true\"/>"
+    "        </property>"
+
     "        <property name=\"displayedLength\" type=\"u\" access=\"read\" />"
     "        <property name=\"historyLength\" type=\"u\" access=\"read\" />"
     "        <property name=\"waitingLength\" type=\"u\" access=\"read\" />"
@@ -327,10 +332,8 @@ static void dbus_cb_dunst_NotificationListHistory(GDBusConnection *connection,
 {
         LOG_D("CMD: Listing all notifications from history");
 
-        GVariant *answer = NULL;
-        GVariantBuilder *builder;
-
-        builder = g_variant_builder_new(G_VARIANT_TYPE("aa{sv}"));
+        GVariantBuilder builder;
+        g_variant_builder_init(&builder, G_VARIANT_TYPE("aa{sv}"));
 
         GList *notification_list = queues_get_history();
 
@@ -340,8 +343,7 @@ static void dbus_cb_dunst_NotificationListHistory(GDBusConnection *connection,
                 n = g_list_nth_data(notification_list, i-1);
 
                 GVariantBuilder n_builder;
-
-                g_variant_builder_init(&n_builder, g_variant_type_new("a{sv}"));
+                g_variant_builder_init(&n_builder, G_VARIANT_TYPE("a{sv}"));
 
                 char *body, *msg, *summary, *appname, *category;
                 char *default_action_name, *icon_path;
@@ -355,49 +357,23 @@ static void dbus_cb_dunst_NotificationListHistory(GDBusConnection *connection,
                         "" : n->default_action_name;
                 icon_path = (n->icon_path == NULL) ? "" : n->icon_path;
 
-                g_variant_builder_add(&n_builder, "{sv}", "body",
-                        g_variant_new_from_bytes(G_VARIANT_TYPE("s"),
-                        g_bytes_new(body, strlen(body)+1), TRUE));
-                g_variant_builder_add(&n_builder, "{sv}", "message",
-                        g_variant_new_from_bytes(G_VARIANT_TYPE("s"),
-                        g_bytes_new(msg, strlen(msg)+1), TRUE));
-                g_variant_builder_add(&n_builder, "{sv}", "summary",
-                        g_variant_new_from_bytes(G_VARIANT_TYPE("s"),
-                        g_bytes_new(summary, strlen(summary)+1), TRUE));
-                g_variant_builder_add(&n_builder, "{sv}", "appname",
-                        g_variant_new_from_bytes(G_VARIANT_TYPE("s"),
-                        g_bytes_new(appname, strlen(appname)+1), TRUE));
-                g_variant_builder_add(&n_builder, "{sv}", "category",
-                        g_variant_new_from_bytes(G_VARIANT_TYPE("s"),
-                        g_bytes_new(category, strlen(category)+1), TRUE));
+                g_variant_builder_add(&n_builder, "{sv}", "body", g_variant_new_string(body));
+                g_variant_builder_add(&n_builder, "{sv}", "message", g_variant_new_string(msg));
+                g_variant_builder_add(&n_builder, "{sv}", "summary", g_variant_new_string(summary));
+                g_variant_builder_add(&n_builder, "{sv}", "appname", g_variant_new_string(appname));
+                g_variant_builder_add(&n_builder, "{sv}", "category", g_variant_new_string(category));
                 g_variant_builder_add(&n_builder, "{sv}", "default_action_name",
-                        g_variant_new_from_bytes(G_VARIANT_TYPE("s"),
-                        g_bytes_new(default_action_name,
-                        strlen(default_action_name)+1), TRUE));
-                g_variant_builder_add(&n_builder, "{sv}", "icon_path",
-                        g_variant_new_from_bytes(G_VARIANT_TYPE("s"),
-                        g_bytes_new(icon_path, strlen(icon_path)+1), TRUE));
-                g_variant_builder_add(&n_builder, "{sv}", "id",
-                        g_variant_new_from_bytes(G_VARIANT_TYPE("i"),
-                        g_bytes_new(&n->id, sizeof(int)), TRUE));
-                g_variant_builder_add(&n_builder, "{sv}", "timestamp",
-                        g_variant_new_from_bytes(G_VARIANT_TYPE("x"),
-                        g_bytes_new(&n->timestamp, sizeof(gint64)), TRUE));
-                g_variant_builder_add(&n_builder, "{sv}", "timeout",
-                        g_variant_new_from_bytes(G_VARIANT_TYPE("x"),
-                        g_bytes_new(&n->timeout, sizeof(gint64)), TRUE));
-                g_variant_builder_add(&n_builder, "{sv}", "progress",
-                        g_variant_new_from_bytes(G_VARIANT_TYPE("i"),
-                        g_bytes_new(&n->progress, sizeof(int)), TRUE));
-
-                g_variant_builder_add(builder, "a{sv}", &n_builder);
+                        g_variant_new_string(default_action_name));
+                g_variant_builder_add(&n_builder, "{sv}", "icon_path", g_variant_new_string(icon_path));
+                g_variant_builder_add(&n_builder, "{sv}", "id", g_variant_new_int32(n->id));
+                g_variant_builder_add(&n_builder, "{sv}", "timestamp", g_variant_new_int64(n->timestamp));
+                g_variant_builder_add(&n_builder, "{sv}", "timeout", g_variant_new_int64(n->timeout));
+                g_variant_builder_add(&n_builder, "{sv}", "progress", g_variant_new_int32(n->progress));
 
+                g_variant_builder_add(&builder, "a{sv}", &n_builder);
         }
 
-        answer = g_variant_new("(aa{sv})", builder);
-
-        g_clear_pointer(&builder, g_variant_builder_unref);
-        g_dbus_method_invocation_return_value(invocation, answer);
+        g_dbus_method_invocation_return_value(invocation, g_variant_new("(aa{sv})", &builder));
         g_dbus_connection_flush(connection, NULL, NULL, NULL);
 }
 
@@ -458,6 +434,7 @@ static void dbus_cb_dunst_RuleEnable(GDBusConnection *connection,
         }
 
         struct rule *target_rule = get_rule(name);
+        g_free(name);
 
         if (target_rule == NULL) {
                 g_dbus_method_invocation_return_error(invocation,
@@ -665,25 +642,30 @@ static struct notification *dbus_message_to_notification(const gchar *sender, GV
 
         // Modify these values after the notification is initialized and all rules are applied.
         if ((dict_value = g_variant_lookup_value(hints, "fgcolor", G_VARIANT_TYPE_STRING))) {
-                g_free(n->colors.fg);
-                n->colors.fg = g_variant_dup_string(dict_value, NULL);
+                struct color c;
+                if (string_parse_color(g_variant_get_string(dict_value, NULL), &c))
+                        n->colors.fg = c;
                 g_variant_unref(dict_value);
         }
 
         if ((dict_value = g_variant_lookup_value(hints, "bgcolor", G_VARIANT_TYPE_STRING))) {
-                g_free(n->colors.bg);
-                n->colors.bg = g_variant_dup_string(dict_value, NULL);
+                struct color c;
+                if (string_parse_color(g_variant_get_string(dict_value, NULL), &c))
+                        n->colors.bg = c;
                 g_variant_unref(dict_value);
         }
 
         if ((dict_value = g_variant_lookup_value(hints, "frcolor", G_VARIANT_TYPE_STRING))) {
-                g_free(n->colors.frame);
-                n->colors.frame = g_variant_dup_string(dict_value, NULL);
+                struct color c;
+                if (string_parse_color(g_variant_get_string(dict_value, NULL), &c))
+                        n->colors.frame = c;
                 g_variant_unref(dict_value);
         }
 
         if ((dict_value = g_variant_lookup_value(hints, "hlcolor", G_VARIANT_TYPE_STRING))) {
-                n->colors.highlight = g_variant_dup_string(dict_value, NULL);
+                struct color c;
+                if (string_parse_color(g_variant_get_string(dict_value, NULL), &c))
+                        n->colors.highlight = c;
                 g_variant_unref(dict_value);
         }
 
@@ -694,7 +676,7 @@ static struct notification *dbus_message_to_notification(const gchar *sender, GV
         return n;
 }
 
-void signal_length_propertieschanged()
+void signal_length_propertieschanged(void)
 {
 
         static unsigned int last_displayed = 0;
@@ -750,7 +732,7 @@ void signal_length_propertieschanged()
                                               "PropertiesChanged",
                                               body,
                                               &err);
-                                
+
                 if (err) {
                         LOG_W("Unable to emit signal: %s", err->message);
                         g_error_free(err);
@@ -923,7 +905,9 @@ GVariant *dbus_cb_dunst_Properties_Get(GDBusConnection *connection,
         struct dunst_status status = dunst_status_get();
 
         if (STR_EQ(property_name, "paused")) {
-                return g_variant_new_boolean(!status.running);
+                return g_variant_new_boolean(status.pause_level != 0);
+        } else if (STR_EQ(property_name, "pauseLevel")) {
+                return g_variant_new_uint32(status.pause_level);
         } else if (STR_EQ(property_name, "displayedLength")) {
                 unsigned int displayed = queues_length_displayed();
                 return g_variant_new_uint32(displayed);
@@ -949,15 +933,32 @@ gboolean dbus_cb_dunst_Properties_Set(GDBusConnection *connection,
                                       GError **error,
                                       gpointer user_data)
 {
+        int targetPauseLevel = -1;
         if (STR_EQ(property_name, "paused")) {
-                dunst_status(S_RUNNING, !g_variant_get_boolean(value));
+                if (g_variant_get_boolean(value)) {
+                        targetPauseLevel = MAX_PAUSE_LEVEL;                 
+                } else {
+                        targetPauseLevel = 0;
+                }
+        } else if STR_EQ(property_name, "pauseLevel") {
+                targetPauseLevel = g_variant_get_uint32(value);
+                if (targetPauseLevel > MAX_PAUSE_LEVEL) {
+                        targetPauseLevel = MAX_PAUSE_LEVEL;
+                }
+        }
+
+        if (targetPauseLevel >= 0) {
+                dunst_status_int(S_PAUSE_LEVEL, targetPauseLevel);
                 wake_up();
 
                 GVariantBuilder *builder = g_variant_builder_new(G_VARIANT_TYPE_VARDICT);
                 GVariantBuilder *invalidated_builder = g_variant_builder_new(G_VARIANT_TYPE_STRING_ARRAY);
                 g_variant_builder_add(builder,
                                       "{sv}",
-                                      "paused", g_variant_new_boolean(g_variant_get_boolean(value)));
+                                      "paused", g_variant_new_boolean(targetPauseLevel != 0));
+                g_variant_builder_add(builder,
+                                      "{sv}",
+                                      "pauseLevel", g_variant_new_uint32(targetPauseLevel));
                 g_dbus_connection_emit_signal(connection,
                                               NULL,
                                               object_path,
