@@ -20,7 +20,7 @@ int dbus_init(void);
 void dbus_teardown(int id);
 void signal_notification_closed(struct notification *n, enum reason reason);
 void signal_action_invoked(const struct notification *n, const char *identifier);
-void signal_length_propertieschanged();
+void signal_length_propertieschanged(void);
 
 #endif
 /* vim: set ft=c tabstop=8 shiftwidth=8 expandtab textwidth=0: */
