@@ -37,11 +37,21 @@ void dunst_status(const enum dunst_status_field field,
         case S_IDLE:
                 status.idle = value;
                 break;
-        case S_RUNNING:
-                status.running = value;
+        default:
+                LOG_E("Invalid %s enum value in %s:%d for bool type", "dunst_status", __FILE__, __LINE__);
+                break;
+        }
+}
+
+void dunst_status_int(const enum dunst_status_field field,
+                  int value)
+{
+        switch (field) {
+        case S_PAUSE_LEVEL:
+                status.pause_level = value;
                 break;
         default:
-                LOG_E("Invalid %s enum value in %s:%d", "dunst_status", __FILE__, __LINE__);
+                LOG_E("Invalid %s enum value in %s:%d for int type", "dunst_status", __FILE__, __LINE__);
                 break;
         }
 }
@@ -55,6 +65,29 @@ struct dunst_status dunst_status_get(void)
 /* misc functions */
 static gboolean run(void *data);
 
+/**
+ * The reason for which the run function was invoked.
+ * - DUNST_TIMER: the timer until the next event in queue has expired (routine
+ *   wakeup)
+ * - DUNST_WAKEUP: an external event (eg. new notification) triggered a call to
+ *   wake_up (unscheduled wakeup)
+ */
+enum dunst_run_reason {
+        DUNST_TIMER,
+        DUNST_WAKEUP,
+};
+
+const char* dunst_run_reason_str(enum dunst_run_reason reason) {
+        switch(reason) {
+                case DUNST_TIMER:
+                        return "DUNST_TIMER";
+                case DUNST_WAKEUP:
+                        return "DUNST_WAKEUP";
+                default:
+                        return "BAD VALUE";
+        }
+}
+
 void wake_up(void)
 {
         // If wake_up is being called before the output has been setup we should
@@ -65,16 +98,39 @@ void wake_up(void)
         }
 
         LOG_D("Waking up");
-        run(GINT_TO_POINTER(1));
+        run(GINT_TO_POINTER(DUNST_WAKEUP));
 }
 
 static gboolean run(void *data)
 {
-        static gint64 next_timeout = 0;
-        static guint next_timeout_id = 0;
-        int reason = GPOINTER_TO_INT(data);
+        /* Timer gestion
+         * =============
+         *
+         * - At any time (except transiently during the execution of `run`), at
+         *   most one glib timeout source (or "timer" here) exists. If `run`
+         *   was invoked by a timer, it will be deleted upon return, as the
+         *   function always returns G_SOURCE_REMOVE.
+         * - Furthermore, if next_timeout_id is not null, a timer with this
+         *   glib source id exists and is running. As a consequence,
+         *   - if reason is DUNST_TIMER, it is the timer that triggered the
+         *     current call to run;
+         *   - if reason is DUNST_WAKEUP, this timer was scheduled some time in
+         *     the future (or in the past, but not yet executed my the main loop,
+         *     which is equivalent for our purpose).
+         *
+         * Thus, in each call to run,
+         * - if reason is DUNST_WAKEUP and next_timeout_id != 0, we delete this
+         *   timer -- we now have more recent information on which we can
+         *   decide of a (maybe) better timeout.
+         * - in any case, we reset next_timeout_id to 0.
+         * - if there is any event to be run in the future, we set a new timer
+         *   to this time, and update next_timeout_id accordingly.
+         */
 
-        LOG_D("RUN, reason %i", reason);
+        static guint next_timeout_id = 0;
+        enum dunst_run_reason reason = GPOINTER_TO_INT(data);
+
+        LOG_D("RUN, reason %i: %s", reason, dunst_run_reason_str(reason));
         gint64 now = time_monotonic_now();
 
         dunst_status(S_FULLSCREEN, output->have_fullscreen_window());
@@ -82,6 +138,12 @@ static gboolean run(void *data)
 
         queues_update(status, now);
 
+        if(reason == DUNST_WAKEUP && next_timeout_id != 0) {
+                // Delete the upcoming timer
+                g_source_remove(next_timeout_id);
+        }
+        next_timeout_id = 0;
+
         if (!queues_length_displayed()) {
                 output->win_hide(win);
                 return G_SOURCE_REMOVE;
@@ -102,15 +164,7 @@ static gboolean run(void *data)
 
                 LOG_D("Sleeping for %li ms", sleep/1000);
 
-                if (sleep >= 0) {
-                        if (reason == 0 || next_timeout < now || timeout_at < next_timeout) {
-                                if (next_timeout != 0) {
-                                        g_source_remove(next_timeout_id);
-                                }
-                                next_timeout_id = g_timeout_add(sleep/1000, run, NULL);
-                                next_timeout = timeout_at;
-                        }
-                }
+                next_timeout_id = g_timeout_add(sleep/1000, run, NULL);
         }
 
         /* If the execution got triggered by g_timeout_add,
@@ -123,7 +177,7 @@ static gboolean run(void *data)
 
 gboolean pause_signal(gpointer data)
 {
-        dunst_status(S_RUNNING, false);
+        dunst_status_int(S_PAUSE_LEVEL, MAX_PAUSE_LEVEL);
         wake_up();
 
         return G_SOURCE_CONTINUE;
@@ -131,7 +185,7 @@ gboolean pause_signal(gpointer data)
 
 gboolean unpause_signal(gpointer data)
 {
-        dunst_status(S_RUNNING, true);
+        dunst_status_int(S_PAUSE_LEVEL, 0);
         wake_up();
 
         return G_SOURCE_CONTINUE;
@@ -156,17 +210,18 @@ static void teardown(void)
 int dunst_main(int argc, char *argv[])
 {
 
-        dunst_status(S_RUNNING, true);
+        dunst_status_int(S_PAUSE_LEVEL, 0);
         dunst_status(S_IDLE, false);
 
+        settings_init();
+
         queues_init();
 
         cmdline_load(argc, argv);
 
         dunst_log_init(false);
 
-        if (cmdline_get_bool("-v/-version", false, "Print version")
-            || cmdline_get_bool("--version", false, "Print version")) {
+        if (cmdline_get_bool("-v/-version/--version", false, "Print version")) {
                 print_version();
         }
 
@@ -178,21 +233,18 @@ int dunst_main(int argc, char *argv[])
         cmdline_config_path =
             cmdline_get_string("-conf/-config", NULL,
                                "Path to configuration file");
-        load_settings(cmdline_config_path);
 
-        if (cmdline_get_bool("-h/-help", false, "Print help")
-            || cmdline_get_bool("--help", false, "Print help")) {
+        settings.print_notifications = cmdline_get_bool("-print/--print", false, "Print notifications to stdout");
+
+        settings.startup_notification = cmdline_get_bool("-startup_notification/--startup_notification",
+                        false, "Display a notification on startup.");
+
+        /* Help should always be the last to set up as calls to cmdline_get_* (as a side effect) add entries to the usage list. */
+        if (cmdline_get_bool("-h/-help/--help", false, "Print help")) {
                 usage(EXIT_SUCCESS);
         }
 
-        if (cmdline_get_bool("-print", false, "Print notifications to stdout")
-            || cmdline_get_bool("--print", false, "Print notifications to stdout")) {
-                settings.print_notifications = true;
-        }
-
-        settings.startup_notification = cmdline_get_bool("--startup_notification",
-                        0, "Display a notification on startup.");
-
+        load_settings(cmdline_config_path);
         int dbus_owner_id = dbus_init();
 
         mainloop = g_main_loop_new(NULL, FALSE);
@@ -223,7 +275,7 @@ int dunst_main(int argc, char *argv[])
         }
 
         setup_done = true;
-        run(NULL);
+        run(GINT_TO_POINTER(DUNST_TIMER)); // The first run() is a scheduled one
         g_main_loop_run(mainloop);
         g_clear_pointer(&mainloop, g_main_loop_unref);
 
