@@ -10,17 +10,19 @@
 
 #include "notification.h"
 
+#define MAX_PAUSE_LEVEL 100
+
 //!< A structure to describe dunst's global window status
 struct dunst_status {
         bool fullscreen; //!< a fullscreen window is currently focused
-        bool running;    //!< set true if dunst is currently running
+        int pause_level;    //!< current pause level. 0 = all notifications come through, 100 = no notifications come through
         bool idle;       //!< set true if the user is idle
 };
 
 enum dunst_status_field {
         S_FULLSCREEN,
         S_IDLE,
-        S_RUNNING,
+        S_PAUSE_LEVEL,
 };
 
 /**
@@ -30,6 +32,8 @@ enum dunst_status_field {
  */
 void dunst_status(const enum dunst_status_field field,
                   bool value);
+void dunst_status_int(const enum dunst_status_field field,
+                  int value);
 
 struct dunst_status dunst_status_get(void);
 
