@@ -185,19 +185,17 @@ static GdkPixbuf *icon_pixbuf_scale_to_size(GdkPixbuf *pixbuf, double dpi_scale,
 
 GdkPixbuf *get_pixbuf_from_file(const char *filename, int min_size, int max_size, double scale)
 {
-        char *path = string_to_path(g_strdup(filename));
         GError *error = NULL;
         gint w, h;
 
-        if (!gdk_pixbuf_get_file_info (path, &w, &h)) {
-                LOG_W("Failed to load image info for %s", filename);
-                g_free(path);
+        if (!gdk_pixbuf_get_file_info (filename, &w, &h)) {
+                LOG_W("Failed to load image info for %s", STR_NN(filename));
                 return NULL;
         }
         GdkPixbuf *pixbuf = NULL;
         // TODO immediately rescale icon upon scale changes
         icon_size_clamp(&w, &h, min_size, max_size);
-        pixbuf = gdk_pixbuf_new_from_file_at_scale(path,
+        pixbuf = gdk_pixbuf_new_from_file_at_scale(filename,
                         round(w * scale),
                         round(h * scale),
                         TRUE,
@@ -208,20 +206,20 @@ GdkPixbuf *get_pixbuf_from_file(const char *filename, int min_size, int max_size
                 g_error_free(error);
         }
 
-        g_free(path);
         return pixbuf;
 }
 
 char *get_path_from_icon_name(const char *iconname, int size)
 {
-        if (settings.enable_recursive_icon_lookup) {
-                char *path = find_icon_path(iconname, size);
-                LOG_I("Found icon at %s", path);
-                return path;
-        }
         if (STR_EMPTY(iconname))
                 return NULL;
 
+        if (settings.enable_recursive_icon_lookup) {
+                char *path = find_icon_path(iconname, size);
+                LOG_I("Found icon at %s", STR_NN(path));
+                return path;
+        }
+
         const char *suffixes[] = { ".svg", ".svgz", ".png", ".xpm", NULL };
         gchar *uri_path = NULL;
         char *new_name = NULL;
@@ -265,7 +263,7 @@ char *get_path_from_icon_name(const char *iconname, int size)
                         start = end + 1;
                 } while (STR_FULL(end));
                 if (!new_name)
-                        LOG_W("No icon found in path: '%s'", iconname);
+                        LOG_W("No icon found in path: '%s'", STR_NN(iconname));
         }
 
         g_free(uri_path);
