@@ -52,24 +52,27 @@ void notification_print(const struct notification *n)
 {
         //TODO: use logging info for this
         printf("{\n");
-        printf("\tappname: '%s'\n", n->appname);
-        printf("\tsummary: '%s'\n", n->summary);
-        printf("\tbody: '%s'\n", n->body);
-        printf("\ticon: '%s'\n", n->iconname);
+        printf("\tappname: '%s'\n", STR_NN(n->appname));
+        printf("\tsummary: '%s'\n", STR_NN(n->summary));
+        printf("\tbody: '%s'\n", STR_NN(n->body));
+        printf("\ticon: '%s'\n", STR_NN(n->iconname));
         printf("\traw_icon set: %s\n", (n->icon_id && !STR_EQ(n->iconname, n->icon_id)) ? "true" : "false");
-        printf("\ticon_id: '%s'\n", n->icon_id);
+        printf("\ticon_id: '%s'\n", STR_NN(n->icon_id));
         printf("\tdesktop_entry: '%s'\n", n->desktop_entry ? n->desktop_entry : "");
-        printf("\tcategory: %s\n", n->category);
+        printf("\tcategory: %s\n", STR_NN(n->category));
         printf("\ttimeout: %ld\n", n->timeout/1000);
+        printf("\tstart: %ld\n", n->start);
+        printf("\ttimestamp: %ld\n", n->timestamp);
         printf("\turgency: %s\n", notification_urgency_to_string(n->urgency));
         printf("\ttransient: %d\n", n->transient);
-        printf("\tformatted: '%s'\n", n->msg);
-        printf("\tfg: %s\n", n->colors.fg);
-        printf("\tbg: %s\n", n->colors.bg);
-        printf("\thighlight: %s\n", n->colors.highlight);
-        printf("\tframe: %s\n", n->colors.frame);
+        printf("\tformatted: '%s'\n", STR_NN(n->msg));
+        char buf[10];
+        printf("\tfg: %s\n", STR_NN(color_to_string(n->colors.fg, buf)));
+        printf("\tbg: %s\n", STR_NN(color_to_string(n->colors.bg, buf)));
+        printf("\thighlight: %s\n", STR_NN(color_to_string(n->colors.highlight, buf)));
+        printf("\tframe: %s\n", STR_NN(color_to_string(n->colors.frame, buf)));
         printf("\tfullscreen: %s\n", enum_to_string_fullscreen(n->fullscreen));
-        printf("\tformat: %s\n", n->format);
+        printf("\tformat: %s\n", STR_NN(n->format));
         printf("\tprogress: %d\n", n->progress);
         printf("\tstack_tag: %s\n", (n->stack_tag ? n->stack_tag : ""));
         printf("\tid: %d\n", n->id);
@@ -77,7 +80,7 @@ void notification_print(const struct notification *n)
                 char *urls = string_replace_all("\n", "\t\t\n", g_strdup(n->urls));
                 printf("\turls:\n");
                 printf("\t{\n");
-                printf("\t\t%s\n", urls);
+                printf("\t\t%s\n", STR_NN(urls));
                 printf("\t}\n");
                 g_free(urls);
         }
@@ -89,14 +92,14 @@ void notification_print(const struct notification *n)
                 g_hash_table_iter_init(&iter, n->actions);
                 printf("\tactions: {\n");
                 while (g_hash_table_iter_next(&iter, &p_key, &p_value))
-                        printf("\t\t\"%s\": \"%s\"\n", (char*)p_key, (char*)p_value);
+                        printf("\t\t\"%s\": \"%s\"\n", (char*)p_key, STR_NN((char*)p_value));
                 printf("\t}\n");
         }
         printf("\tscript_count: %d\n", n->script_count);
         if (n->script_count > 0) {
                 printf("\tscripts: ");
                 for (int i = 0; i < n->script_count; i++) {
-                        printf("'%s' ",n->scripts[i]);
+                        printf("'%s' ", STR_NN(n->scripts[i]));
                 }
                 printf("\n");
         }
@@ -195,11 +198,29 @@ const char *notification_urgency_to_string(const enum urgency urgency)
 /* see notification.h */
 int notification_cmp(const struct notification *a, const struct notification *b)
 {
-        if (settings.sort && a->urgency != b->urgency) {
-                return b->urgency - a->urgency;
+        const struct notification *a_order;
+        const struct notification *b_order;
+        if(settings.sort == SORT_TYPE_UPDATE && settings.origin & ZWLR_LAYER_SURFACE_V1_ANCHOR_BOTTOM){
+                a_order = b;
+                b_order = a;
         } else {
-                return a->id - b->id;
+                a_order = a;
+                b_order = b;
         }
+
+        if(settings.sort == SORT_TYPE_URGENCY_ASCENDING){
+                if(a_order->urgency != b_order->urgency){
+                        return a_order->urgency - b_order->urgency;
+                }
+        } else if (settings.sort == SORT_TYPE_URGENCY_DESCENDING) {
+                if(a_order->urgency != b_order->urgency){
+                        return b_order->urgency - a_order->urgency;
+                }
+        } else if(settings.sort == SORT_TYPE_UPDATE){
+                return b_order->timestamp - a_order->timestamp;
+        }
+
+        return a_order->id - b_order->id;
 }
 
 /* see notification.h */
@@ -283,10 +304,6 @@ void notification_unref(struct notification *n)
         g_free(n->category);
         g_free(n->text_to_render);
         g_free(n->urls);
-        g_free(n->colors.fg);
-        g_free(n->colors.bg);
-        g_free(n->colors.highlight);
-        g_free(n->colors.frame);
         g_free(n->stack_tag);
         g_free(n->desktop_entry);
 
@@ -320,11 +337,11 @@ void notification_transfer_icon(struct notification *from, struct notification *
 
 void notification_icon_replace_path(struct notification *n, const char *new_icon)
 {
-        ASSERT_OR_RET(n,);
+        ASSERT_OR_RET(n && n->icon_position != ICON_OFF,);
         ASSERT_OR_RET(new_icon,);
-        if(n->iconname && n->icon && strcmp(n->iconname, new_icon) == 0) {
+
+        if (n->iconname && n->icon && STR_EQ(n->iconname, new_icon))
                 return;
-        }
 
         // make sure it works, even if n->iconname is passed as new_icon
         if (n->iconname != new_icon) {
@@ -346,14 +363,14 @@ void notification_icon_replace_path(struct notification *n, const char *new_icon
                         n->icon = gdk_pixbuf_to_cairo_surface(pixbuf);
                         g_object_unref(pixbuf);
                 } else {
-                        LOG_W("No icon found in path: '%s'", n->icon_path);
+                        LOG_W("Failed to load icon from path: '%s'", n->icon_path);
                 }
         }
 }
 
 void notification_icon_replace_data(struct notification *n, GVariant *new_icon)
 {
-        ASSERT_OR_RET(n,);
+        ASSERT_OR_RET(n && n->icon_position != ICON_OFF,);
         ASSERT_OR_RET(new_icon,);
 
         cairo_surface_destroy(n->icon);
@@ -429,6 +446,12 @@ struct notification *notification_create(void)
         n->max_icon_size = 32;
         n->receiving_raw_icon = false;
 
+        struct color invalid = COLOR_UNINIT;
+        n->colors.fg = invalid;
+        n->colors.bg = invalid;
+        n->colors.highlight = invalid;
+        n->colors.frame = invalid;
+
         n->script_run = false;
         n->dbus_valid = false;
 
@@ -475,19 +498,17 @@ void notification_init(struct notification *n)
                 default:
                         g_error("Unhandled urgency type: %d", n->urgency);
         }
-        if (!n->colors.fg)
-                n->colors.fg = g_strdup(defcolors.fg);
-        if (!n->colors.bg)
-                n->colors.bg = g_strdup(defcolors.bg);
-        if (!n->colors.highlight)
-                n->colors.highlight = g_strdup(defcolors.highlight);
-        if (!n->colors.frame)
-                n->colors.frame = g_strdup(defcolors.frame);
+        if (!COLOR_VALID(n->colors.fg)) n->colors.fg = defcolors.fg;
+        if (!COLOR_VALID(n->colors.bg)) n->colors.bg = defcolors.bg;
+        if (!COLOR_VALID(n->colors.highlight)) n->colors.highlight = defcolors.highlight;
+        if (!COLOR_VALID(n->colors.frame)) n->colors.frame = defcolors.frame;
 
         /* Sanitize misc hints */
         if (n->progress < 0)
                 n->progress = -1;
 
+        n->override_pause_level = 0;
+
         /* Process rules */
         rule_apply_all(n);
 
@@ -705,7 +726,7 @@ void notification_update_text_to_render(struct notification *n)
 void notification_do_action(struct notification *n)
 {
         assert(n->default_action_name);
-        
+
         if (g_hash_table_size(n->actions)) {
                 if (g_hash_table_contains(n->actions, n->default_action_name)) {
                         signal_action_invoked(n, n->default_action_name);
