@@ -8,6 +8,7 @@
 #include <cairo.h>
 
 #include "markup.h"
+#include "draw.h"
 
 #define DUNST_NOTIF_MAX_CHARS 50000
 
@@ -38,10 +39,10 @@ enum urgency {
 typedef struct _notification_private NotificationPrivate;
 
 struct notification_colors {
-        char *frame;
-        char *bg;
-        char *fg;
-        char *highlight;
+        struct color frame;
+        struct color bg;
+        struct color fg;
+        struct color highlight;
 };
 
 struct notification {
@@ -56,6 +57,7 @@ struct notification {
         char *category;
         char *desktop_entry;     /**< The desktop entry hint sent via every GApplication */
         enum urgency urgency;
+        int override_pause_level;
 
         cairo_surface_t *icon;         /**< The raw cached icon data used to draw */
         char *icon_id;           /**< Plain icon information, which acts as the icon's id.
