@@ -120,7 +120,10 @@ static void queues_swap_notifications(GQueue *queueA,
  */
 static bool queues_notification_is_ready(const struct notification *n, struct dunst_status status, bool shown)
 {
-        ASSERT_OR_RET(status.running, false);
+        if (status.pause_level > n->override_pause_level) {
+                return false;
+        }
+
         if (status.fullscreen && shown)
                 return n && n->fullscreen != FS_PUSHBACK;
         else if (status.fullscreen && !shown)
@@ -172,7 +175,7 @@ int queues_notification_insert(struct notification *n)
                 if (settings.always_run_script) {
                         notification_run_script(n);
                 }
-                LOG_M("Skipping notification: '%s' '%s'", n->body, n->summary);
+                LOG_M("Skipping notification: '%s' '%s'", STR_NN(n->body), STR_NN(n->summary));
                 return 0;
         }
 
@@ -190,8 +193,12 @@ int queues_notification_insert(struct notification *n)
         if (!inserted && STR_FULL(n->stack_tag) && queues_stack_by_tag(n))
                 inserted = true;
 
-        if (!inserted && settings.stack_duplicates && queues_stack_duplicate(n))
+        if (!inserted && settings.stack_duplicates && queues_stack_duplicate(n)){
+                if(settings.sort == SORT_TYPE_UPDATE){
+                        g_queue_sort(displayed, notification_cmp_data, NULL);
+                }
                 inserted = true;
+        }
 
         if (!inserted)
                 g_queue_insert_sorted(waiting, n, notification_cmp_data, NULL);
@@ -368,7 +375,7 @@ void queues_history_pop_by_id(unsigned int id)
         if (g_queue_is_empty(history))
                 return;
 
-        // search through the history buffer 
+        // search through the history buffer
         for (GList *iter = g_queue_peek_head_link(history); iter;
                 iter = iter->next) {
                 struct notification *cur = iter->data;
