@@ -59,22 +59,14 @@ void rule_apply(struct rule *r, struct notification *n)
                 n->markup = r->markup;
         if (r->icon_position != -1)
                 n->icon_position = r->icon_position;
-        if (r->fg) {
-                g_free(n->colors.fg);
-                n->colors.fg = g_strdup(r->fg);
-        }
-        if (r->bg) {
-                g_free(n->colors.bg);
-                n->colors.bg = g_strdup(r->bg);
-        }
-        if (r->highlight) {
-                g_free(n->colors.highlight);
-                n->colors.highlight = g_strdup(r->highlight);
-        }
-        if (r->fc) {
-                g_free(n->colors.frame);
-                n->colors.frame = g_strdup(r->fc);
-        }
+        if (COLOR_VALID(r->fg))
+                n->colors.fg = r->fg;
+        if (COLOR_VALID(r->bg))
+                n->colors.bg = r->bg;
+        if (COLOR_VALID(r->highlight))
+                n->colors.highlight = r->highlight;
+        if (COLOR_VALID(r->fc))
+                n->colors.frame = r->fc;
         if (r->format)
                 n->format = r->format;
         if (r->default_icon) {
@@ -99,6 +91,9 @@ void rule_apply(struct rule *r, struct notification *n)
                 g_free(n->stack_tag);
                 n->stack_tag = g_strdup(r->set_stack_tag);
         }
+        if (r->override_pause_level != -1) {
+                n->override_pause_level = r->override_pause_level;
+        }
 }
 
 /*
@@ -171,11 +166,11 @@ static inline bool rule_field_matches_string(const char *value, const char *patt
                         char *err_buf = g_malloc(err_size);
                         regerror(err, &regex, err_buf, err_size);
                         LOG_W("%s: \"%s\"", err_buf, pattern);
-                        free(err_buf);
+                        g_free(err_buf);
                         return false;
                 }
 
-                for (int i = 0; ; i++) {
+                while (true) {
                         if (regexec(&regex, value, 0, NULL, 0))
                                 break;
                         regfree(&regex);
