@@ -45,12 +45,13 @@ struct rule {
         int icon_position;
         int min_icon_size;
         int max_icon_size;
+        int override_pause_level;
         char *new_icon;
-        char *fg;
-        char *bg;
-        char *highlight;
         char *default_icon;
-        char *fc;
+        struct color fg;
+        struct color bg;
+        struct color highlight;
+        struct color fc;
         char *set_category;
         const char *format;
         const char *script;
