@@ -20,7 +20,6 @@
 #include "ini.h"
 #include "rules.h"
 #include "utils.h"
-#include "x11/x.h"
 #include "output.h"
 
 #ifndef SYSCONFDIR
@@ -60,7 +59,7 @@ static int is_drop_in(const struct dirent *dent) {
  * The result @e must @e not be freed! The array is cached in a static variable,
  * so it is OK to call this again instead of caching its return value.
  */
-static GPtrArray *get_xdg_conf_basedirs() {
+static GPtrArray *get_xdg_conf_basedirs(void) {
         GPtrArray *arr = g_ptr_array_new_full(4, g_free);
         g_ptr_array_add(arr, g_build_filename(g_get_user_config_dir(), "dunst", NULL));
 
@@ -96,9 +95,9 @@ static void config_files_add_drop_ins(GPtrArray *config_files, const char *path)
                                 drop_ins[n]->d_name, NULL);
                 LOG_D("Found drop-in: %s\n", drop_in);
                 g_ptr_array_insert(config_files, insert_index, drop_in);
-                free(drop_ins[n]);
+                g_free(drop_ins[n]);
         }
-        free(drop_ins);
+        g_free(drop_ins);
 }
 
 /** @brief Find all config files.
@@ -140,18 +139,18 @@ static GPtrArray* get_conf_files(const char *path) {
 FILE *fopen_conf(char * const path)
 {
         FILE *f = NULL;
-        char *real_path = string_to_path(strdup(path));
+        char *real_path = string_to_path(g_strdup(path));
 
         if (is_readable_file(real_path) && (f = fopen(real_path, "r")))
                 LOG_I(MSG_FOPEN_SUCCESS(path, f));
         else
                 LOG_W(MSG_FOPEN_FAILURE(path));
 
-        free(real_path);
+        g_free(real_path);
         return f;
 }
 
-void settings_init() {
+void settings_init(void) {
         static bool init_done = false;
         if (!init_done) {
                 LOG_D("Initializing settings");
@@ -161,12 +160,16 @@ void settings_init() {
         }
 }
 
+// NOTE: This function is probably outdated and no longer relevant
+//       Since it does not even fully display rule information we
+//       may want to remove it or change it
 void print_rule(struct rule* r) {
-        LOG_D("Rule %s", r->name);
-        LOG_D("summary %s", r->summary);
-        LOG_D("appname %s", r->appname);
-        LOG_D("script %s", r->script);
-        LOG_D("frame %s", r->fc);
+        LOG_D("Rule %s", STR_NN(r->name));
+        LOG_D("summary %s", STR_NN(r->summary));
+        LOG_D("appname %s", STR_NN(r->appname));
+        LOG_D("script %s", STR_NN(r->script));
+        char buf[10];
+        LOG_D("frame %s", STR_NN(color_to_string(r->fc, buf)));
 }
 
 void check_and_correct_settings(struct settings *s) {
@@ -256,12 +259,15 @@ static void process_conf_file(const gpointer conf_fname, gpointer n_success) {
         check_and_correct_settings(&settings);
 
         finish_ini(ini);
-        free(ini);
+        g_free(ini);
 
         ++(*(int *) n_success);
 }
 
-void load_settings(const char * const path) {
+void load_settings(const char *const path) {
+        // NOTE: settings_init should be called before if some settings are changed
+        //       But having it here is useful for tests
+        //       Potentially, we could update the settings code and move this somewhere else
         settings_init();
         LOG_D("Setting defaults");
         set_defaults();
