@@ -112,7 +112,7 @@ struct setting {
 /*
  *   How to add/change a rule
  *   ------------------------
- * 
+ *
  * - Add variable to `struct rules` in `rules.h` (make sure to read the comment
  *   at the top of the struct)
  * - Add variable to to `struct notification` in `notification.h`
@@ -156,15 +156,18 @@ static const struct rule empty_rule = {
         .alignment       = -1,
         .hide_text       = -1,
         .new_icon        = NULL,
-        .fg              = NULL,
-        .bg              = NULL,
-        .format          = NULL,
         .default_icon    = NULL,
+        .fg              = COLOR_UNINIT,
+        .bg              = COLOR_UNINIT,
+        .highlight       = COLOR_UNINIT,
+        .fc              = COLOR_UNINIT,
+        .format          = NULL,
         .script          = NULL,
         .enabled         = true,
         .progress_bar_alignment   = -1,
         .min_icon_size   = -1,
         .max_icon_size   = -1,
+        .override_pause_level = -1
 };
 
 
@@ -222,6 +225,32 @@ static const struct string_to_enum_def boolean_enum_data[] = {
         ENUM_END,
 };
 
+static const struct string_to_enum_def sort_type_enum_data[] = {
+        {"True", SORT_TYPE_URGENCY_DESCENDING },
+        {"true", SORT_TYPE_URGENCY_DESCENDING },
+        {"On", SORT_TYPE_URGENCY_DESCENDING },
+        {"on", SORT_TYPE_URGENCY_DESCENDING },
+        {"Yes", SORT_TYPE_URGENCY_DESCENDING },
+        {"yes", SORT_TYPE_URGENCY_DESCENDING },
+        {"1", SORT_TYPE_URGENCY_DESCENDING },
+        {"False", SORT_TYPE_ID },
+        {"false", SORT_TYPE_ID },
+        {"Off", SORT_TYPE_ID },
+        {"off", SORT_TYPE_ID },
+        {"No", SORT_TYPE_ID },
+        {"no", SORT_TYPE_ID },
+        {"0", SORT_TYPE_ID },
+        {"n", SORT_TYPE_ID },
+        {"y", SORT_TYPE_ID },
+        {"N", SORT_TYPE_ID },
+        {"Y", SORT_TYPE_URGENCY_DESCENDING },
+        {"id",                  SORT_TYPE_ID},
+        {"urgency_ascending",   SORT_TYPE_URGENCY_ASCENDING },
+        {"urgency_descending",  SORT_TYPE_URGENCY_DESCENDING },
+        {"update",  SORT_TYPE_UPDATE },
+        ENUM_END,
+};
+
 static const struct string_to_enum_def horizontal_alignment_enum_data[] = {
         {"left",   PANGO_ALIGN_LEFT },
         {"center", PANGO_ALIGN_CENTER },
@@ -318,6 +347,20 @@ static const struct string_to_enum_def origin_enum_data[] = {
         ENUM_END,
 };
 
+static const struct string_to_enum_def corners_enum_data[] = {
+        { "top-left", C_TOP_LEFT },
+        { "top-right", C_TOP_RIGHT },
+        { "bottom-left", C_BOT_LEFT },
+        { "bottom-right", C_BOT_RIGHT },
+        { "left", C_LEFT },
+        { "right", C_RIGHT },
+        { "bottom", C_BOT },
+        { "top", C_TOP },
+        { "all", C_ALL },
+        { "none", C_NONE },
+        ENUM_END,
+};
+
 static const struct setting allowed_settings[] = {
         // These icon settings have to be above the icon rule
         {
@@ -478,7 +521,7 @@ static const struct setting allowed_settings[] = {
                 .name = "background",
                 .section = "*",
                 .description = "The background color of the notification.",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "*",
                 .value = NULL,
                 .parser = NULL,
@@ -500,7 +543,7 @@ static const struct setting allowed_settings[] = {
                 .name = "foreground",
                 .section = "*",
                 .description = "The foreground color of the notification.",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "*",
                 .value = NULL,
                 .parser = NULL,
@@ -511,7 +554,7 @@ static const struct setting allowed_settings[] = {
                 .name = "highlight",
                 .section = "*",
                 .description = "The highlight color of the notification.",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "*",
                 .value = NULL,
                 .parser = NULL,
@@ -522,7 +565,7 @@ static const struct setting allowed_settings[] = {
                 .name = "default_icon",
                 .section = "*",
                 .description = "The default icon that is used when no icon is passed",
-                .type = TYPE_STRING,
+                .type = TYPE_PATH,
                 .default_value = "*",
                 .value = NULL,
                 .parser = NULL,
@@ -727,6 +770,36 @@ static const struct setting allowed_settings[] = {
                 .parser_data = boolean_enum_data,
                 .rule_offset = offsetof(struct rule, enabled),
         },
+        {
+                .name = "corners",
+                .section = "global",
+                .description = "Select the corners to round",
+                .type = TYPE_CUSTOM,
+                .default_value = "all",
+                .value = &settings.corners,
+                .parser = string_parse_corners,
+                .parser_data = corners_enum_data,
+        },
+        {
+                .name = "progress_bar_corners",
+                .section = "global",
+                .description = "Select the corners to round for the progress bar",
+                .type = TYPE_CUSTOM,
+                .default_value = "all",
+                .value = &settings.progress_bar_corners,
+                .parser = string_parse_corners,
+                .parser_data = corners_enum_data,
+        },
+        {
+                .name = "icon_corners",
+                .section = "global",
+                .description = "Select the corners to round for the icon image",
+                .type = TYPE_CUSTOM,
+                .default_value = "all",
+                .value = &settings.icon_corners,
+                .parser = string_parse_corners,
+                .parser_data = corners_enum_data,
+        },
         {
                 .name = "progress_bar_horizontal_alignment",
                 .section = "*",
@@ -760,6 +833,17 @@ static const struct setting allowed_settings[] = {
                 .parser_data = NULL,
                 .rule_offset = offsetof(struct rule, max_icon_size),
         },
+        {
+                .name = "override_pause_level",
+                .section = "*",
+                .description = "TODO",
+                .type = TYPE_INT,
+                .default_value = "-1",
+                .value = NULL,
+                .parser = NULL,
+                .parser_data = NULL,
+                .rule_offset = offsetof(struct rule, override_pause_level),
+        },
         // end of modifying rules
 
         // other settings below
@@ -767,7 +851,7 @@ static const struct setting allowed_settings[] = {
                 .name = "frame_color",
                 .section = "*",
                 .description = "Color of the frame around the window",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "#888888",
                 .value = &settings.frame_color,
                 .parser = NULL,
@@ -817,12 +901,12 @@ static const struct setting allowed_settings[] = {
         {
                 .name = "sort",
                 .section = "global",
-                .description = "Sort notifications by urgency and date?",
+                .description = "Sort type by id/urgency/update",
                 .type = TYPE_CUSTOM,
                 .default_value = "true",
                 .value = &settings.sort,
                 .parser = string_parse_enum,
-                .parser_data = boolean_enum_data,
+                .parser_data = sort_type_enum_data,
         },
         {
                 .name = "indicate_hidden",
@@ -868,11 +952,11 @@ static const struct setting allowed_settings[] = {
                 .name = "monitor",
                 .section = "global",
                 .description = "On which monitor should the notifications be displayed",
-                .type = TYPE_INT,
+                .type = TYPE_CUSTOM,
                 .default_value = "0",
                 .value = &settings.monitor,
-                .parser = NULL,
-                .parser_data = NULL,
+                .parser = string_parse_maybe_int,
+                .parser_data = &settings.monitor_num,
         },
         {
                 .name = "title",
@@ -1274,7 +1358,7 @@ static const struct setting allowed_settings[] = {
                 .name = "background",
                 .section = "urgency_low",
                 .description = "Background color for notifications with low urgency",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "#222222",
                 .value = &settings.colors_low.bg,
                 .parser = NULL,
@@ -1284,7 +1368,7 @@ static const struct setting allowed_settings[] = {
                 .name = "foreground",
                 .section = "urgency_low",
                 .description = "Foreground color for notifications with low urgency",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "#888888",
                 .value = &settings.colors_low.fg,
                 .parser = NULL,
@@ -1294,7 +1378,7 @@ static const struct setting allowed_settings[] = {
                 .name = "highlight",
                 .section = "urgency_low",
                 .description = "Highlight color for notifications with low urgency",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "#7f7fff",
                 .value = &settings.colors_low.highlight,
                 .parser = NULL,
@@ -1304,7 +1388,7 @@ static const struct setting allowed_settings[] = {
                 .name = "frame_color",
                 .section = "urgency_low",
                 .description = "Frame color for notifications with low urgency",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "#888888",
                 .value = &settings.colors_low.frame,
                 .parser = NULL,
@@ -1324,7 +1408,7 @@ static const struct setting allowed_settings[] = {
                 .name = "background",
                 .section = "urgency_normal",
                 .description = "Background color for notifications with normal urgency",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "#285577",
                 .value = &settings.colors_norm.bg,
                 .parser = NULL,
@@ -1334,7 +1418,7 @@ static const struct setting allowed_settings[] = {
                 .name = "foreground",
                 .section = "urgency_normal",
                 .description = "Foreground color for notifications with normal urgency",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "#ffffff",
                 .value = &settings.colors_norm.fg,
                 .parser = NULL,
@@ -1344,7 +1428,7 @@ static const struct setting allowed_settings[] = {
                 .name = "highlight",
                 .section = "urgency_normal",
                 .description = "Highlight color for notifications with normal urgency",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "#1745d1",
                 .value = &settings.colors_norm.highlight,
                 .parser = NULL,
@@ -1354,7 +1438,7 @@ static const struct setting allowed_settings[] = {
                 .name = "frame_color",
                 .section = "urgency_normal",
                 .description = "Frame color for notifications with normal urgency",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "#888888",
                 .value = &settings.colors_norm.frame,
                 .parser = NULL,
@@ -1374,7 +1458,7 @@ static const struct setting allowed_settings[] = {
                 .name = "background",
                 .section = "urgency_critical",
                 .description = "Background color for notifications with critical urgency",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "#900000",
                 .value = &settings.colors_crit.bg,
                 .parser = NULL,
@@ -1384,7 +1468,7 @@ static const struct setting allowed_settings[] = {
                 .name = "foreground",
                 .section = "urgency_critical",
                 .description = "Foreground color for notifications with ciritical urgency",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "#ffffff",
                 .value = &settings.colors_crit.fg,
                 .parser = NULL,
@@ -1394,7 +1478,7 @@ static const struct setting allowed_settings[] = {
                 .name = "highlight",
                 .section = "urgency_critical",
                 .description = "Highlight color for notifications with ciritical urgency",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "#ff6666",
                 .value = &settings.colors_crit.highlight,
                 .parser = NULL,
@@ -1404,7 +1488,7 @@ static const struct setting allowed_settings[] = {
                 .name = "frame_color",
                 .section = "urgency_critical",
                 .description = "Frame color for notifications with critical urgency",
-                .type = TYPE_STRING,
+                .type = TYPE_COLOR,
                 .default_value = "#ff0000",
                 .value = &settings.colors_crit.frame,
                 .parser = NULL,
