@@ -24,247 +24,24 @@
 #include "protocols/wlr-layer-shell-unstable-v1.h"
 #include "protocols/wlr-foreign-toplevel-management-unstable-v1-client-header.h"
 #include "protocols/wlr-foreign-toplevel-management-unstable-v1.h"
-#include "protocols/idle-client-header.h"
-#include "protocols/idle.h"
 #include "pool-buffer.h"
 
-
+#include "../dunst.h"
 #include "../log.h"
 #include "../settings.h"
-#include "../queues.h"
-#include "../input.h"
 #include "libgwater-wayland.h"
 #include "foreign_toplevel.h"
+#include "wl_ctx.h"
 #include "wl_output.h"
-
-#define MAX_TOUCHPOINTS 10
+#include "wl_seat.h"
 
 struct window_wl {
         cairo_surface_t *c_surface;
         cairo_t * c_ctx;
 };
 
-struct wl_ctx {
-        GWaterWaylandSource *esrc;
-        struct wl_display *display; // owned by esrc
-        struct wl_registry *registry;
-        struct wl_compositor *compositor;
-        struct wl_shm *shm;
-        struct zwlr_layer_shell_v1 *layer_shell;
-        struct wl_seat *seat;
-
-        struct wl_list outputs;
-
-        struct wl_surface *surface;
-        struct dunst_output *surface_output;
-        struct zwlr_layer_surface_v1 *layer_surface;
-        struct dunst_output *layer_surface_output;
-        struct wl_callback *frame_callback;
-        struct org_kde_kwin_idle *idle_handler;
-        struct org_kde_kwin_idle_timeout *idle_timeout;
-        uint32_t toplevel_manager_name;
-        struct zwlr_foreign_toplevel_manager_v1 *toplevel_manager;
-        bool configured;
-        bool dirty;
-        bool is_idle;
-        bool has_idle_monitor;
-
-        struct {
-                struct wl_pointer *wl_pointer;
-                int32_t x, y;
-        } pointer;
-
-        struct {
-                struct wl_touch *wl_touch;
-                struct {
-                        int32_t x, y;
-                } pts[MAX_TOUCHPOINTS];
-        } touch;
-
-        struct dimensions cur_dim;
-
-        int32_t width, height;
-        struct pool_buffer buffers[2];
-        struct pool_buffer *current_buffer;
-        struct wl_cursor_theme *cursor_theme;
-        const struct wl_cursor_image *cursor_image;
-        struct wl_surface *cursor_surface;
-};
-
 struct wl_ctx ctx;
 
-static void noop() {
-        // This space intentionally left blank
-}
-
-void set_dirty();
-
-static void output_handle_geometry(void *data, struct wl_output *wl_output,
-                int32_t x, int32_t y, int32_t phy_width, int32_t phy_height,
-                int32_t subpixel, const char *make, const char *model,
-                int32_t transform) {
-        //TODO do something with the subpixel data
-        struct dunst_output *output = data;
-        output->subpixel = subpixel;
-}
-static void output_handle_mode(void *data, struct wl_output *wl_output, uint32_t flags,
-                int32_t width, int32_t height, int32_t refresh) {
-        struct dunst_output *output = data;
-        output->width = width;
-        output->height = height;
-}
-
-static void output_handle_scale(void *data, struct wl_output *wl_output,
-                int32_t factor) {
-        struct dunst_output *output = data;
-        output->scale = factor;
-
-        wake_up();
-}
-
-static const struct wl_output_listener output_listener = {
-        .geometry = output_handle_geometry,
-        .mode = output_handle_mode,
-        .done = noop,
-        .scale = output_handle_scale,
-};
-
-static void create_output( struct wl_output *wl_output, uint32_t global_name) {
-        struct dunst_output *output = g_malloc0(sizeof(struct dunst_output));
-        if (output == NULL) {
-                LOG_E("allocation failed");
-                return;
-        }
-
-        bool recreate_surface = false;
-        static int number = 0;
-        LOG_I("New output found - id %i", number);
-        output->global_name = global_name;
-        output->wl_output = wl_output;
-        output->scale = 1;
-        output->fullscreen = false;
-
-        recreate_surface = wl_list_empty(&ctx.outputs);
-
-        wl_list_insert(&ctx.outputs, &output->link);
-
-        wl_output_set_user_data(wl_output, output);
-        wl_output_add_listener(wl_output, &output_listener, output);
-        number++;
-
-        if (recreate_surface) {
-                // We had no outputs, force our surface to redraw
-                set_dirty(ctx.surface);
-        }
-}
-
-static void destroy_output(struct dunst_output *output) {
-        if (ctx.surface_output == output) {
-                ctx.surface_output = NULL;
-        }
-        if (ctx.layer_surface_output == output) {
-                ctx.layer_surface_output = NULL;
-        }
-        wl_list_remove(&output->link);
-        wl_output_destroy(output->wl_output);
-        free(output->name);
-        free(output);
-}
-
-static void touch_handle_motion(void *data, struct wl_touch *wl_touch,
-                uint32_t time, int32_t id,
-                wl_fixed_t surface_x, wl_fixed_t surface_y) {
-        if (id >= MAX_TOUCHPOINTS) {
-                return;
-        }
-        ctx.touch.pts[id].x = wl_fixed_to_int(surface_x);
-        ctx.touch.pts[id].y = wl_fixed_to_int(surface_y);
-}
-
-static void touch_handle_down(void *data, struct wl_touch *wl_touch,
-                uint32_t serial, uint32_t time, struct wl_surface *sfc, int32_t id,
-                wl_fixed_t surface_x, wl_fixed_t surface_y) {
-        if (id >= MAX_TOUCHPOINTS) {
-                return;
-        }
-        ctx.touch.pts[id].x = wl_fixed_to_int(surface_x);
-        ctx.touch.pts[id].y = wl_fixed_to_int(surface_y);
-}
-
-static void touch_handle_up(void *data, struct wl_touch *wl_touch,
-                uint32_t serial, uint32_t time, int32_t id) {
-        if (id >= MAX_TOUCHPOINTS) {
-                return;
-        }
-        input_handle_click(BTN_TOUCH, false,
-                        ctx.touch.pts[id].x, ctx.touch.pts[id].y);
-
-}
-
-static void pointer_handle_enter(void *data, struct wl_pointer *wl_pointer, uint32_t serial, struct wl_surface *surface, wl_fixed_t x, wl_fixed_t y) {
-        // Change the mouse cursor to "left_ptr"
-        if (ctx.cursor_theme != NULL) {
-                wl_pointer_set_cursor(wl_pointer, serial, ctx.cursor_surface, ctx.cursor_image->hotspot_x, ctx.cursor_image->hotspot_y);
-        }
-}
-
-static void pointer_handle_motion(void *data, struct wl_pointer *wl_pointer,
-                uint32_t time, wl_fixed_t surface_x, wl_fixed_t surface_y) {
-        ctx.pointer.x = wl_fixed_to_int(surface_x);
-        ctx.pointer.y = wl_fixed_to_int(surface_y);
-}
-
-static void pointer_handle_button(void *data, struct wl_pointer *wl_pointer,
-                uint32_t serial, uint32_t time, uint32_t button,
-                uint32_t button_state) {
-        input_handle_click(button, button_state, ctx.pointer.x, ctx.pointer.y);
-}
-
-static const struct wl_pointer_listener pointer_listener = {
-        .enter = pointer_handle_enter,
-        .leave = noop,
-        .motion = pointer_handle_motion,
-        .button = pointer_handle_button,
-        .axis = noop,
-};
-
-static const struct wl_touch_listener touch_listener = {
-        .down = touch_handle_down,
-        .up = touch_handle_up,
-        .motion = touch_handle_motion,
-        .frame = noop,
-        .cancel = noop,
-};
-
-static void seat_handle_capabilities(void *data, struct wl_seat *wl_seat,
-                uint32_t capabilities) {
-
-        if (ctx.pointer.wl_pointer != NULL) {
-                wl_pointer_release(ctx.pointer.wl_pointer);
-                ctx.pointer.wl_pointer = NULL;
-        }
-        if (capabilities & WL_SEAT_CAPABILITY_POINTER) {
-                ctx.pointer.wl_pointer = wl_seat_get_pointer(wl_seat);
-                wl_pointer_add_listener(ctx.pointer.wl_pointer,
-                        &pointer_listener, ctx.seat);
-        }
-        if (ctx.touch.wl_touch != NULL) {
-                wl_touch_release(ctx.touch.wl_touch);
-                ctx.touch.wl_touch = NULL;
-        }
-        if (capabilities & WL_SEAT_CAPABILITY_TOUCH) {
-                ctx.touch.wl_touch = wl_seat_get_touch(wl_seat);
-                wl_touch_add_listener(ctx.touch.wl_touch,
-                        &touch_listener, ctx.seat);
-        }
-}
-
-static const struct wl_seat_listener seat_listener = {
-        .capabilities = seat_handle_capabilities,
-        .name = noop,
-};
-
-
 static void surface_handle_enter(void *data, struct wl_surface *surface,
                 struct wl_output *wl_output) {
         // Don't bother keeping a list of outputs, a layer surface can only be on
@@ -284,8 +61,8 @@ static const struct wl_surface_listener surface_listener = {
 };
 
 
-static void schedule_frame_and_commit();
-static void send_frame();
+static void schedule_frame_and_commit(void);
+static void send_frame(void);
 
 static void layer_surface_handle_configure(void *data,
                 struct zwlr_layer_surface_v1 *surface,
@@ -339,37 +116,11 @@ static const struct zwlr_layer_surface_v1_listener layer_surface_listener = {
         .closed = layer_surface_handle_closed,
 };
 
-
-static void idle_start (void *data, struct org_kde_kwin_idle_timeout *org_kde_kwin_idle_timeout) {
-        ctx.is_idle = true;
-        LOG_D("User went idle");
-}
-static void idle_stop (void *data, struct org_kde_kwin_idle_timeout *org_kde_kwin_idle_timeout) {
-        ctx.is_idle = false;
-        LOG_D("User isn't idle anymore");
-}
-
-static const struct org_kde_kwin_idle_timeout_listener idle_timeout_listener = {
-        .idle = idle_start,
-        .resumed = idle_stop,
-};
-
-static void add_seat_to_idle_handler(struct wl_seat *seat) {
-        if (!ctx.idle_handler) {
-                return;
-        }
-        if (settings.idle_threshold > 0) {
-                uint32_t timeout_ms = settings.idle_threshold/1000;
-                ctx.idle_timeout = org_kde_kwin_idle_get_idle_timeout(ctx.idle_handler, seat, timeout_ms);
-                org_kde_kwin_idle_timeout_add_listener(ctx.idle_timeout, &idle_timeout_listener, 0);
-                ctx.has_idle_monitor = true;
-        }
-}
-
 // Warning, can return NULL
-static struct dunst_output *get_configured_output() {
+static struct dunst_output *get_configured_output(void) {
         int n = 0;
-        int target_monitor = settings.monitor;
+        int target_monitor = settings.monitor_num;
+        const char *name = settings.monitor;
 
         struct dunst_output *first_output = NULL, *configured_output = NULL,
                             *tmp_output = NULL;
@@ -378,6 +129,8 @@ static struct dunst_output *get_configured_output() {
                         first_output = tmp_output;
                 if (n == target_monitor)
                         configured_output = tmp_output;
+                if (g_strcmp0(name, tmp_output->name) == 0)
+                        configured_output = tmp_output;
                 n++;
         }
 
@@ -386,9 +139,9 @@ static struct dunst_output *get_configured_output() {
                 return first_output;
 
         switch (settings.f_mode){
-                case FOLLOW_NONE: ; // this semicolon is neccesary
+                case FOLLOW_NONE:
                         if (!configured_output) {
-                                LOG_W("Monitor %i doesn't exist, using focused monitor", settings.monitor);
+                                LOG_W("Screen '%s' not found, using focused monitor", settings.monitor);
                         }
                         return configured_output;
                 case FOLLOW_MOUSE:
@@ -412,17 +165,29 @@ static void handle_global(void *data, struct wl_registry *registry,
                 ctx.layer_shell = wl_registry_bind(registry, name,
                                 &zwlr_layer_shell_v1_interface, 1);
         } else if (strcmp(interface, wl_seat_interface.name) == 0) {
-                ctx.seat = wl_registry_bind(registry, name, &wl_seat_interface, 3);
-                wl_seat_add_listener(ctx.seat, &seat_listener, ctx.seat);
-                add_seat_to_idle_handler(ctx.seat);
+                create_seat(registry, name, version);
+                LOG_D("Binding to seat %i", name);
         } else if (strcmp(interface, wl_output_interface.name) == 0) {
-                struct wl_output *output =
-                        wl_registry_bind(registry, name, &wl_output_interface, 3);
-                create_output(output, name);
+                create_output(registry, name, version);
                 LOG_D("Binding to output %i", name);
         } else if (strcmp(interface, org_kde_kwin_idle_interface.name) == 0 &&
                         version >= ORG_KDE_KWIN_IDLE_TIMEOUT_IDLE_SINCE_VERSION) {
+#ifdef HAVE_WL_EXT_IDLE_NOTIFY
+                if (ctx.ext_idle_notifier)
+                        return;
+#endif
                 ctx.idle_handler = wl_registry_bind(registry, name, &org_kde_kwin_idle_interface, 1);
+                LOG_D("Found org_kde_kwin_idle");
+#ifdef HAVE_WL_EXT_IDLE_NOTIFY
+        } else if (strcmp(interface, ext_idle_notifier_v1_interface.name) == 0) {
+                ctx.ext_idle_notifier = wl_registry_bind(registry, name, &ext_idle_notifier_v1_interface, 1);
+                LOG_D("Found ext-idle-notify-v1");
+#endif
+#ifdef HAVE_WL_CURSOR_SHAPE
+        } else if (strcmp(interface, wp_cursor_shape_manager_v1_interface.name) == 0) {
+                ctx.cursor_shape_manager = wl_registry_bind(registry, name,
+                                &wp_cursor_shape_manager_v1_interface, 1);
+#endif
         } else if (strcmp(interface, zwlr_foreign_toplevel_manager_v1_interface.name) == 0 &&
                         version >= ZWLR_FOREIGN_TOPLEVEL_HANDLE_V1_STATE_FULLSCREEN_SINCE_VERSION) {
                 LOG_D("Found toplevel manager %i", name);
@@ -436,7 +201,15 @@ static void handle_global_remove(void *data, struct wl_registry *registry,
         wl_list_for_each_safe(output, tmp, &ctx.outputs, link) {
                 if (output->global_name == name) {
                         destroy_output(output);
-                        break;
+                        return;
+                }
+        }
+
+        struct dunst_seat *seat, *tmp_seat;
+        wl_list_for_each_safe(seat, tmp_seat, &ctx.seats, link) {
+                if (seat->global_name == name) {
+                        destroy_seat(seat);
+                        return;
                 }
         }
 }
@@ -446,10 +219,10 @@ static const struct wl_registry_listener registry_listener = {
         .global_remove = handle_global_remove,
 };
 
-bool wl_init() {
+bool wl_init(void) {
         wl_list_init(&ctx.outputs);
+        wl_list_init(&ctx.seats);
         wl_list_init(&toplevel_list);
-        //wl_list_init(&ctx.seats); // TODO multi-seat support
 
         ctx.esrc = g_water_wayland_source_new(NULL, NULL);
         ctx.display = g_water_wayland_source_get_display(ctx.esrc);
@@ -492,15 +265,19 @@ bool wl_init() {
                 LOG_W("compositor doesn't support zwlr_layer_shell_v1");
                 return false;
         }
-        if (ctx.seat == NULL) {
+        if (wl_list_empty(&ctx.seats)) {
                 LOG_W("no seat was found, so dunst cannot see input");
+        } else if (ctx.idle_handler == NULL
+#ifdef HAVE_WL_EXT_IDLE_NOTIFY
+                && ctx.ext_idle_notifier == NULL
+#endif
+        ) {
+                LOG_I("compositor doesn't support org_kde_kwin_idle or ext-idle-notify-v1");
         } else {
-                if (ctx.idle_handler == NULL) {
-                        LOG_I("compositor doesn't support org_kde_kwin_idle_interface");
-                }
-                else if (ctx.idle_timeout == NULL && settings.idle_threshold > 0) {
-                        // something went wrong in setting the timeout
-                        LOG_W("couldn't set idle timeout");
+                // Set up idle monitoring for any seats that we received before the idle protocols
+                struct dunst_seat *seat = NULL;
+                wl_list_for_each(seat, &ctx.seats, link) {
+                        add_seat_to_idle_handler(seat);
                 }
         }
 
@@ -528,9 +305,14 @@ bool wl_init() {
                 LOG_W("couldn't find a cursor theme");
                 return true;
         }
-        struct wl_cursor *cursor = wl_cursor_theme_get_cursor(ctx.cursor_theme, "left_ptr");
+        // Try cursor spec (CSS) name first
+        struct wl_cursor *cursor = wl_cursor_theme_get_cursor(ctx.cursor_theme, "default");
         if (cursor == NULL) {
-                LOG_W("couldn't find cursor icon \"left_ptr\"");
+                // Try legacy Xcursor name
+                cursor = wl_cursor_theme_get_cursor(ctx.cursor_theme, "left_ptr");
+        }
+        if (cursor == NULL) {
+                LOG_W("couldn't find cursor icons \"default\" or \"left_ptr\"");
                 wl_cursor_theme_destroy(ctx.cursor_theme);
                 // Set to NULL so it doesn't get free'd again
                 ctx.cursor_theme = NULL;
@@ -545,7 +327,7 @@ bool wl_init() {
         return true;
 }
 
-void wl_deinit() {
+void wl_deinit(void) {
         // We need to check if any of these are NULL, since the initialization
         // could have been aborted half way through, or the compositor doesn't
         // support some of these features.
@@ -565,19 +347,24 @@ void wl_deinit() {
                 destroy_output(output);
         }
 
-        if (ctx.seat) {
-                if (ctx.pointer.wl_pointer)
-                        wl_pointer_release(ctx.pointer.wl_pointer);
-                wl_seat_release(ctx.seat);
-                ctx.seat = NULL;
+        struct dunst_seat *seat, *seat_tmp;
+        wl_list_for_each_safe(seat, seat_tmp, &ctx.seats, link) {
+                destroy_seat(seat);
         }
 
+#ifdef HAVE_WL_CURSOR_SHAPE
+        if (ctx.cursor_shape_manager)
+                wp_cursor_shape_manager_v1_destroy(ctx.cursor_shape_manager);
+#endif
+
+#ifdef HAVE_WL_EXT_IDLE_NOTIFY
+        if (ctx.ext_idle_notifier)
+                ext_idle_notifier_v1_destroy(ctx.ext_idle_notifier);
+#endif
+
         if (ctx.idle_handler)
                 org_kde_kwin_idle_destroy(ctx.idle_handler);
 
-        if (ctx.idle_timeout)
-                org_kde_kwin_idle_timeout_release(ctx.idle_timeout);
-
         if (ctx.layer_shell)
                 zwlr_layer_shell_v1_destroy(ctx.layer_shell);
 
@@ -599,10 +386,10 @@ void wl_deinit() {
         g_water_wayland_source_free(ctx.esrc);
 }
 
-static void schedule_frame_and_commit();
+static void schedule_frame_and_commit(void);
 
 // Draw and commit a new frame.
-static void send_frame() {
+static void send_frame(void) {
         int scale = wl_get_scale();
 
         if (wl_list_empty(&ctx.outputs)) {
@@ -749,7 +536,7 @@ static const struct wl_callback_listener frame_listener = {
         .done = frame_handle_done,
 };
 
-static void schedule_frame_and_commit() {
+static void schedule_frame_and_commit(void) {
         if (ctx.frame_callback) {
                 return;
         }
@@ -763,7 +550,7 @@ static void schedule_frame_and_commit() {
         wl_surface_commit(ctx.surface);
 }
 
-void set_dirty() {
+void set_dirty(void) {
         if (ctx.dirty) {
                 return;
         }
@@ -856,14 +643,20 @@ const struct screen_info* wl_get_active_screen(void) {
 }
 
 bool wl_is_idle(void) {
-        LOG_I("Idle status queried: %i", ctx.is_idle);
+        struct dunst_seat *seat = NULL;
+        bool is_idle = true;
         // When the user doesn't have a seat, or their compositor doesn't support the idle
         // protocol, we'll assume that they are not idle.
-        if (settings.idle_threshold == 0 || ctx.has_idle_monitor == false) {
-                return false;
+        if (settings.idle_threshold == 0 || wl_list_empty(&ctx.seats)) {
+                is_idle = false;
         } else {
-                return ctx.is_idle;
+                wl_list_for_each(seat, &ctx.seats, link) {
+                        // seat.is_idle cannot be set without an idle protocol
+                        is_idle &= seat->is_idle;
+                }
         }
+        LOG_I("Idle status queried: %i", is_idle);
+        return is_idle;
 }
 
 bool wl_have_fullscreen_window(void) {
