@@ -139,7 +139,7 @@ TEST test_string_to_int(void)
         struct setting s;
         s.type = TYPE_INT;
 
-        char buf[50];
+        static char buf[50];
         for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
                 sprintf(buf, "Failed in round %i", i);
                 ASSERTm(buf, set_from_string(&val, s, inputs[i]));
@@ -166,7 +166,7 @@ TEST test_string_to_int_invalid(void)
         s.type = TYPE_INT;
         s.name = "test_int";
 
-        char buf[50];
+        static char buf[50];
         for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
                 sprintf(buf, "Failed in round %i", i);
                 ASSERT_FALSEm(buf, set_from_string(&val, s, inputs[i]));
@@ -199,7 +199,7 @@ TEST test_string_to_double(void)
         struct setting s;
         s.type = TYPE_DOUBLE;
 
-        char buf[50];
+        static char buf[50];
         for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
                 sprintf(buf, "Failed in round %i", i);
                 ASSERTm(buf, set_from_string(&val, s, inputs[i]));
@@ -225,7 +225,7 @@ TEST test_string_to_double_invalid(void)
         s.type = TYPE_DOUBLE;
         s.name = "test_double";
 
-        char buf[50];
+        static char buf[50];
         for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
                 sprintf(buf, "Failed in round %i", i);
                 ASSERT_FALSEm(buf, set_from_string(&val, s, inputs[i]));
@@ -263,7 +263,7 @@ TEST test_string_to_boolean(void)
 
         ARRAY_SAME_LENGTH(inputs, results);
 
-        char buf[50];
+        static char buf[50];
         for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
                 sprintf(buf, "Failed in round %i", i);
                 ASSERTm(buf, set_from_string(&val, s, inputs[i]));
@@ -291,7 +291,7 @@ TEST test_string_to_boolean_invalid(void)
                 "else",
         };
 
-        char buf[50];
+        static char buf[50];
 
         for (int i = 0; i < G_N_ELEMENTS(invalid_inputs); i++) {
                 sprintf(buf, "Failed in round %i", i);
@@ -312,7 +312,7 @@ TEST test_string_to_enum(void)
         s.parser = string_parse_enum;
         s.parser_data = ellipsize_enum_data;
 
-        char buf[50];
+        static char buf[50];
 
         // do not go until last element, since it's ENUM_END (all 0)
         for (int i = 0; i < G_N_ELEMENTS(ellipsize_enum_data)-1; i++) {
@@ -342,7 +342,7 @@ TEST test_string_to_enum_invalid(void)
                 "else"
         };
 
-        char buf[50];
+        static char buf[50];
 
         for (int i = 0; i < G_N_ELEMENTS(invalid_inputs); i++) {
                 sprintf(buf, "Failed in round %i", i);
@@ -383,7 +383,7 @@ TEST test_string_to_list(void)
                 {MOUSE_CLOSE_ALL, MOUSE_CLOSE_CURRENT, MOUSE_CLOSE_ALL, MOUSE_ACTION_END},
         };
 
-        char buf[50];
+        static char buf[50];
         for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
                 sprintf(buf, "Failed in round %i", i);
                 ASSERTm(buf, set_from_string(s.value, s, inputs[i]));
@@ -425,7 +425,7 @@ TEST test_string_to_list_invalid(void)
                 "close_all,invalid,close_current,close_all",
         };
 
-        char buf[256];
+        static char buf[256];
 
         for (int i = 0; i < G_N_ELEMENTS(invalid_inputs); i++) {
                 sprintf(buf, "Failed in round %i", i);
@@ -465,7 +465,7 @@ TEST test_string_to_time(void)
 
         ARRAY_SAME_LENGTH(inputs, results);
 
-        char buf[50];
+        static char buf[50];
         for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
                 sprintf(buf, "Failed in round %i", i);
                 ASSERTm(buf, set_from_string(&val, s, inputs[i]));
@@ -495,7 +495,7 @@ TEST test_string_to_time_invalid(void)
                 "s",
         };
 
-        char buf[50];
+        static char buf[50];
         for (int i = 0; i < G_N_ELEMENTS(invalid_inputs); i++) {
                 sprintf(buf, "Failed in round %i", i);
                 ASSERT_FALSEm(buf, set_from_string(&val, s, invalid_inputs[i]));
@@ -521,9 +521,14 @@ TEST test_string_to_path(void)
                 "/path/p argument",
                 "p with multiple arguments",
                 "~/p/p",
+                "$HOME/p/p",
+                "$TEST_ENV/p/p",
         };
 
+        setenv("TEST_ENV", "foobar", 1);
+
         char *expanded_home = g_strconcat(user_get_home(), "/", "p/p", NULL);
+        char *expanded_env = g_strconcat("foobar", "/p/p", NULL);
         const char* results[] = {
                 "/bin/something",
                 "something",
@@ -531,6 +536,8 @@ TEST test_string_to_path(void)
                 "/path/p argument",
                 "p with multiple arguments",
                 expanded_home,
+                expanded_home,
+                expanded_env,
         };
 
         const char* results2[][5] = {
@@ -540,12 +547,14 @@ TEST test_string_to_path(void)
                 {"/path/p", "argument", NULL},
                 {"p", "with", "multiple", "arguments", NULL},
                 {expanded_home},
+                {expanded_home},
+                {expanded_env},
         };
 
         ARRAY_SAME_LENGTH(inputs, results);
         ARRAY_SAME_LENGTH(inputs, results2);
 
-        char buf[256];
+        static char buf[256];
         for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
                 sprintf(buf, "Failed in round %i", i);
                 ASSERTm(buf, set_from_string(&val, s, inputs[i]));
@@ -557,6 +566,7 @@ TEST test_string_to_path(void)
         }
 
         g_free(val);
+        g_free(expanded_env);
         g_free(expanded_home);
         g_strfreev(val2);
         PASS();
@@ -582,32 +592,35 @@ TEST test_string_to_sepcolor(void)
         };
 
         const struct separator_color_data results[] = {
-                {SEP_AUTO, NULL},
-                {SEP_FOREGROUND, NULL},
-                {SEP_FRAME, NULL},
-                {SEP_CUSTOM, "#123456"},
-                {SEP_CUSTOM, "#ab123c"},
-                {SEP_CUSTOM, "#AB123C"},
+                {SEP_AUTO, COLOR_UNINIT},
+                {SEP_FOREGROUND, COLOR_UNINIT},
+                {SEP_FRAME, COLOR_UNINIT},
+                {SEP_CUSTOM, { (double)0x12 / 0xff, (double)0x34 / 0xff, (double)0x56 / 0xff, 1.0}},
+                {SEP_CUSTOM, { (double)0xab / 0xff, (double)0x12 / 0xff, (double)0x3c / 0xff, 1.0}},
+                {SEP_CUSTOM, { (double)0xab / 0xff, (double)0x12 / 0xff, (double)0x3c / 0xff, 1.0}},
         };
 
         ARRAY_SAME_LENGTH(inputs, results);
 
-        char buf[50];
+        static char buf[100];
+        char buf1[10], buf2[10];
+
         for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
                 sprintf(buf, "Failed in round %i. Expected %i, got %i", i, results[i].type, val.type);
                 ASSERTm(buf, set_from_string(&val, s, inputs[i]));
                 ASSERT_EQm(buf, results[i].type, val.type);
-                sprintf(buf, "Failed in round %i. Expected %s, got %s", i, results[i].sep_color, val.sep_color);
-                ASSERTm(buf, STR_EQ(results[i].sep_color, val.sep_color));
+
+                sprintf(buf, "Failed in round %i. Expected %s, got %s", i,
+                                color_to_string(results[i].color, buf1), color_to_string(val.color, buf2));
+                ASSERTm(buf, (!COLOR_VALID(val.color) && !COLOR_VALID(results[i].color)) || COLOR_SAME(results[i].color, val.color));
         }
 
-        g_free(val.sep_color);
         PASS();
 }
 
 TEST test_string_to_sepcolor_invalid(void)
 {
-        struct separator_color_data val = {123, "test123"};
+        struct separator_color_data val = {123, COLOR_UNINIT};
         struct setting s;
         s.type = TYPE_CUSTOM;
         s.value = &val;
@@ -621,21 +634,96 @@ TEST test_string_to_sepcolor_invalid(void)
                 "fraame",
                 "123456",
                 "#ab",
-
-                // TODO detect these mistakes as well
-                /* "#12456", */
-                /* "#gg123c", */
-                /* "#AB123C123212", */
+                "#gg123c",
+                "#AB123C123212",
         };
 
-        char buf[50];
+        static char buf[50];
         for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
                 sprintf(buf, "Failed in round %i.", i);
                 ASSERT_FALSEm(buf, set_from_string(&val, s, inputs[i]));
         }
 
         ASSERT_EQm("Sep color shouldn't changed from invalid inputs", 123, (int) val.type);
-        ASSERT_STR_EQm("Sep color shouldn't changed from invalid inputs", "test123", val.sep_color);
+        ASSERTm("Sep color shouldn't changed from invalid inputs", !COLOR_VALID(val.color));
+        PASS();
+}
+
+TEST test_string_to_color(void)
+{
+        struct color val = COLOR_UNINIT;
+        struct setting s;
+        s.type = TYPE_COLOR;
+        s.value = &val;
+        s.name = "test_color";
+
+        const char* inputs[] = {
+                "#123456",
+                "#ab123c",
+                "#AB123C",
+                "#abc",
+                "#faf",
+                "#AABBCC10",
+        };
+
+        const struct color results[] = {
+                { (double)0x12 / 0xff, (double)0x34 / 0xff, (double)0x56 / 0xff, 1.0},
+                { (double)0xab / 0xff, (double)0x12 / 0xff, (double)0x3c / 0xff, 1.0},
+                { (double)0xab / 0xff, (double)0x12 / 0xff, (double)0x3c / 0xff, 1.0},
+                { (double)0xaa / 0xff, (double)0xbb / 0xff, (double)0xcc / 0xff, 1.0},
+                { 1.0, (double)0xaa / 0xff, 1.0, 1.0},
+                { (double)0xaa / 0xff, (double)0xbb / 0xff, (double)0xcc / 0xff, (double)0x10 / 0xff},
+        };
+
+        ARRAY_SAME_LENGTH(inputs, results);
+
+        static char buf[100];
+        char buf1[10], buf2[10];
+
+        for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
+                sprintf(buf, "Failed in round %i. Expected %s, got %s", i,
+                                color_to_string(results[i], buf1), color_to_string(val, buf2));
+                ASSERTm(buf, set_from_string(&val, s, inputs[i]));
+                ASSERTm(buf, COLOR_SAME(results[i], val));
+        }
+
+        PASS();
+}
+
+TEST test_string_to_color_invalid(void)
+{
+        struct color val = COLOR_UNINIT;
+        struct setting s;
+        s.type = TYPE_COLOR;
+        s.value = &val;
+        s.name = "test_color";
+
+        const char* inputs[] = {
+                "",
+                "not color",
+                "####",
+                "@#4x",
+                "#abx",
+                "#gg123c",
+                "#AB123C123212",
+                "#sjdsnc",
+                "#00#",
+                "#ab+cd",
+                "#fffffffffffffffff",
+                "   ##   ",
+                "#xzky",
+                "#abacgg",
+                "0xfaf",
+                "#fcb+fa",
+        };
+
+        static char buf[50];
+        for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
+                sprintf(buf, "Failed in round %i.", i);
+                ASSERT_FALSEm(buf, set_from_string(&val, s, inputs[i]));
+        }
+
+        ASSERTm("Color shouldn't changed from invalid inputs", !COLOR_VALID(val));
         PASS();
 }
 
@@ -672,7 +760,7 @@ TEST test_string_to_length(void)
 
         ARRAY_SAME_LENGTH(inputs, results);
 
-        char buf[500];
+        static char buf[500];
         for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
                 sprintf(buf, "Failed in round %i.", i);
                 ASSERTm(buf, set_from_string(&val, s, inputs[i]));
@@ -716,7 +804,7 @@ TEST test_string_to_length_invalid(void)
                 "(123, 122)", // invalid order
         };
 
-        char buf[50];
+        static char buf[50];
         for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
                 sprintf(buf, "Failed in round %i.", i);
                 ASSERT_FALSEm(buf, set_from_string(&val, s, inputs[i]));
@@ -727,6 +815,125 @@ TEST test_string_to_length_invalid(void)
         PASS();
 }
 
+TEST test_string_to_corners(void)
+{
+        enum corner_pos corners = C_NONE;
+
+        struct setting s;
+        s.type = TYPE_CUSTOM;
+        s.value = &corners;
+        s.parser = string_parse_corners;
+        s.parser_data = corners_enum_data;
+
+        static char buf[50];
+
+        // do not go until last element, since it's ENUM_END (all 0)
+        for (int i = 0; i < G_N_ELEMENTS(ellipsize_enum_data)-1; i++) {
+                sprintf(buf, "Failed in round %i", i);
+                ASSERTm(buf, set_from_string(&corners, s, corners_enum_data[i].string));
+                ASSERT_EQm(buf, corners, corners_enum_data[i].enum_value);
+        }
+
+        const char* inputs[] = {
+                "bottom,right",
+                "top-left, top-right",
+                "all",
+                "right,left,top,bottom",
+                "bottom-left,top-right",
+                "all,all", // still accepted
+                "all,top-right",
+        };
+        const enum corner_pos results[] = {
+                C_BOT | C_RIGHT,
+                C_TOP_LEFT | C_TOP_RIGHT,
+                C_ALL,
+                C_ALL,
+                C_BOT_LEFT | C_TOP_RIGHT,
+                C_ALL,
+                C_ALL,
+        };
+
+        ARRAY_SAME_LENGTH(inputs, results);
+
+        for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
+                sprintf(buf, "Failed in round %i", i);
+                ASSERTm(buf, set_from_string(&corners, s, inputs[i]));
+                ASSERT_EQm(buf, corners, results[i]);
+        }
+        PASS();
+}
+
+TEST test_string_to_corners_invalid(void)
+{
+        enum corner_pos corners = C_ALL + 1;
+
+        struct setting s = {0};
+        s.type = TYPE_CUSTOM;
+        s.parser = string_parse_corners;
+        s.parser_data = corners_enum_data;
+        s.value = &corners;
+        s.name = "test_corners";
+
+        const char* invalid_inputs[] = {
+                "123",
+                "something",
+                "else",
+                "al l",
+                "right;right",
+                "bot-left",
+                "top right, bottom left"
+        };
+
+        static char buf[50];
+        for (int i = 0; i < G_N_ELEMENTS(invalid_inputs); i++) {
+                sprintf(buf, "Failed in round %i", i);
+                bool success = set_from_string(&corners, s, invalid_inputs[i]);
+                ASSERT_FALSEm(buf, success);
+        }
+        ASSERT_EQm("Enum should not change from invalid values", corners, C_ALL + 1);
+        PASS();
+}
+
+TEST test_string_to_maybe_int(void)
+{
+        char *val = NULL;
+        int intval;
+        struct setting s;
+        s.type = TYPE_CUSTOM;
+        s.value = &val;
+        s.name = "test_maybe_int";
+        s.parser = string_parse_maybe_int;
+        s.parser_data = &intval;
+
+        const char* inputs[] = {
+                "0",
+                "1",
+                "HDMI-0",
+                "0TEST",
+        };
+
+        const struct { const char *s; int i; } results[] = {
+                { "0", 0 },
+                { "1", 1 },
+                { "HDMI-0", INT_MIN },
+                { "0TEST", INT_MIN },
+        };
+        static char buf[500];
+
+        for (int i = 0; i < G_N_ELEMENTS(inputs); i++) {
+                sprintf(buf, "Failed in round %i.", i);
+                ASSERTm(buf, set_from_string(&val, s, inputs[i]));
+                sprintf(buf, "Failed in round %i. Expected val to be %s, got %s", i, results[i].s, val);
+                ASSERTm(buf, STR_EQ(val, results[i].s));
+                sprintf(buf, "Failed in round %i. Expected intval to be %i, got %i", i, results[i].i, intval);
+                ASSERT_EQm(buf, results[i].i, intval);
+        }
+
+        g_free(val);
+
+        PASS();
+}
+
 #define TEST_ENUM(t) { \
 ASSERT_EQ(sizeof(t), sizeof(int)); \
 }
@@ -743,6 +950,7 @@ TEST test_enum_size(void)
         TEST_ENUM(enum follow_mode);
         TEST_ENUM(enum mouse_action );
         TEST_ENUM(enum zwlr_layer_shell_v1_layer);
+        TEST_ENUM(enum corner_pos);
         PASS();
 }
 
@@ -785,12 +993,17 @@ SUITE(suite_option_parser)
         RUN_TEST(test_string_to_time);
         RUN_TEST(test_string_to_time_invalid);
         RUN_TEST(test_string_to_path);
+        RUN_TEST(test_string_to_corners);
+        RUN_TEST(test_string_to_corners_invalid);
         // Paths are now almost always considered valid
         RUN_TEST(test_string_to_sepcolor);
         RUN_TEST(test_string_to_sepcolor_invalid);
+        RUN_TEST(test_string_to_color);
+        RUN_TEST(test_string_to_color_invalid);
         RUN_TEST(test_enum_size);
         RUN_TEST(test_string_to_length);
         RUN_TEST(test_string_to_length_invalid);
+        RUN_TEST(test_string_to_maybe_int);
 
         g_strfreev(argv);
 }
