@@ -825,10 +825,47 @@ TEST test_queues_update_paused(void)
         PASS();
 }
 
+TEST test_queues_update_pause_level(void)
+{
+        settings.notification_limit = 5;
+        struct notification *n1, *n2, *n3;
+        queues_init();
+
+        n1 = test_notification("n1", 0);
+        n2 = test_notification("n2", 0);
+        n3 = test_notification("n3", 0);
+
+        n1->override_pause_level = 0;
+        n2->override_pause_level = 5;
+        n3->override_pause_level = 10;
+
+        queues_notification_insert(n1);
+        queues_notification_insert(n2);
+        queues_notification_insert(n3);
+
+        queues_update(STATUS_PAUSE_7, time_monotonic_now());
+        QUEUE_LEN_ALL(2,1,0);
+        ASSERT(strcmp(((struct notification*) g_queue_peek_nth(QUEUE(DISP), 0))->summary, "n3") == 0);
+        ASSERT(strcmp(((struct notification*) g_queue_peek_nth(QUEUE(WAIT), 0))->summary, "n1") == 0);
+        ASSERT(strcmp(((struct notification*) g_queue_peek_nth(QUEUE(WAIT), 1))->summary, "n2") == 0);
+
+        queues_update(STATUS_NORMAL, time_monotonic_now());
+        QUEUE_LEN_ALL(0,3,0);
+
+        queues_update(STATUS_PAUSE_7, time_monotonic_now());
+        QUEUE_LEN_ALL(2,1,0);
+        ASSERT(strcmp(((struct notification*) g_queue_peek_nth(QUEUE(DISP), 0))->summary, "n3") == 0);
+        ASSERT(strcmp(((struct notification*) g_queue_peek_nth(QUEUE(WAIT), 0))->summary, "n1") == 0);
+        ASSERT(strcmp(((struct notification*) g_queue_peek_nth(QUEUE(WAIT), 1))->summary, "n2") == 0);
+
+        queues_teardown();
+        PASS();
+}
+
 TEST test_queues_update_seeping(void)
 {
         settings.notification_limit = 5;
-        settings.sort = true;
+        settings.sort = SORT_TYPE_URGENCY_DESCENDING;
         settings.indicate_hidden = false;
         struct notification *nl1, *nl2, *nl3, *nl4, *nl5;
         struct notification *nc1, *nc2, *nc3, *nc4, *nc5;
@@ -1024,7 +1061,7 @@ TEST test_queue_get_history(void)
 }
 
 
-void print_queues() {
+void print_queues(void) {
         printf("\nQueues:\n");
         for (GList *iter = g_queue_peek_head_link(QUEUE_WAIT); iter;
                         iter = iter->next) {
@@ -1105,6 +1142,7 @@ SUITE(suite_queues)
         RUN_TEST(test_queue_insert_id_replacement);
         RUN_TEST(test_queue_insert_id_valid_newid);
         RUN_TEST(test_queue_length);
+        
         RUN_TEST(test_queue_notification_close);
         RUN_TEST(test_queue_notification_close_histignore);
         RUN_TEST(test_queue_notification_skip_display);
@@ -1118,6 +1156,8 @@ SUITE(suite_queues)
         RUN_TEST(test_queue_timeout);
         RUN_TEST(test_queues_update_fullscreen);
         RUN_TEST(test_queues_update_paused);
+        RUN_TEST(test_queues_update_pause_level
+        );
         RUN_TEST(test_queues_update_seep_showlowurg);
         RUN_TEST(test_queues_update_seeping);
         RUN_TEST(test_queues_update_xmore);
